﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Windows.Automation;

using System.IO;
using System.Diagnostics;

namespace EncompassAutoStart
{
    public partial class Form1 : Form
    {
        // params from command line
        string sEXE = "";       // -e - location of Encompass EXE.  Alternatively, you can put EncompassAutoStart in Encompass.exe dir
        string sServer = "";    // -s - server.  Can be https://TEBE12345678.ea.elliemae.net$TEBE12345678 or just TEBE12345678
        string sUID = "";       // -u - user id
        string sPWD = "";       // -p - password


        // Encompass process that we start
        Process pENCOMPASS = null;

        private bool ClientIDDialog_Handled = false;
        private bool LoginDialog_Handled = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
                lblError.Text = "";

                // parse params from command-line
                sEXE = Utils2.ParseCommandLineParam("-e");
                sServer = Utils2.ParseCommandLineParam("-s");
                sUID = Utils2.ParseCommandLineParam("-u");
                sPWD = Utils2.ParseCommandLineParam("-p");

                // adjust server from full https if it was specified as full
                int i1 = sServer.LastIndexOf("$");
                if (i1 > 0)
                {
                    sServer = sServer.Substring(i1 + 1);
                }

                if (String.IsNullOrEmpty(sServer) ||
                    String.IsNullOrEmpty(sUID))
                {
                    throw new Exception("Please specify -s and -u parameters.  -e and -p are optional.");
                }
                else
                {
                    lblStarting.Text = "Starting Encompass " + sServer + " as " + sUID;
                }
            }
            catch (Exception ex)
            {
                Utils2.MsgBox_ErrorOK("Error: " + ex.Message);
                this.Close();
            }
        }

        private void Form1_Shown(object sender, EventArgs e)
        {
            try
            {
                StartEncompassExe();

                // start checking for Login window
                timerCheck.Enabled = true;
            }
            catch (Exception ex)
            {
                Utils2.MsgBox_ErrorOK("Error: " + ex.Message);
                this.Close();
            }
        }

        private void StartEncompassExe()
        {
            // if EXE was not specified - use current directory
            if (String.IsNullOrEmpty(sEXE))
            {
                sEXE = Path.Combine(Utils2.AppPath(), "Encompass.exe");
            }

            if (!File.Exists(sEXE))
            {
                throw new Exception("File not found: " + sEXE);
            }

            ProcessStartInfo si = new ProcessStartInfo();
            si.FileName = sEXE;

            // version
            FileVersionInfo info = FileVersionInfo.GetVersionInfo(sEXE);
            string s = info.FileVersion;
            if (s.StartsWith("18") ||
                s.StartsWith("19") ||
                s.StartsWith("20.0") ||
                s.StartsWith("20.1"))
            {
                // old version - pass command-line params
                si.Arguments = "-s https://" + sServer + ".ea.elliemae.net$" + sServer + " -u " + sUID;
                if (!String.IsNullOrEmpty(sPWD))
                {
                    si.Arguments += " -p " + sPWD;
                }
            }
            else
            {
                // Encompass 20.2 and above don't use commpand-line
            }

            si.CreateNoWindow = false;
            si.UseShellExecute = true;
            si.WindowStyle = ProcessWindowStyle.Normal;
            pENCOMPASS = Process.Start(si);
        }

        private void timerCheck_Tick(object sender, EventArgs e)
        {
            try
            {
                timerCheck.Enabled = false;

                // sanity check
                if (pENCOMPASS == null)
                {
                    this.Close();
                    return;
                }

                pENCOMPASS.Refresh();

                // sanity check
                if (pENCOMPASS.HasExited)
                {
                    this.Close();
                    return;
                }

                IntPtr ptr = pENCOMPASS.MainWindowHandle;
                if (ptr != IntPtr.Zero)
                {
                    AutomationElement frm = AutomationElement.FromHandle(ptr);

                    if (CheckForClientIDDialog(frm))
                    {
                        // first dialog handled - continue to the next one
                    }
                    else if (CheckForLoginDialog(frm))
                    {
                        // success - we just logged in - stop
                        this.Close();
                        return;
                    }
                    else if (CheckForMainWindow(frm))
                    {
                        // we somehow got to main window - stop
                        this.Close();
                        return;
                    }
                }

                // fallthrough - restart timer again
                timerCheck.Enabled = true;
            }
            catch (Exception ex)
            {
                timerCheck.Enabled = false;
                Utils2.MsgBox_ErrorOK("Error: " + ex.Message);
                this.Close();
            }
        }

        private bool CheckForClientIDDialog(AutomationElement frm)
        {
            // don't throw errors
            try
            {
                if (!ClientIDDialog_Handled)
                {
                    if (frm.Current.AutomationId.Equals("AuthenticationForm", StringComparison.OrdinalIgnoreCase) &&
                        frm.Current.Name.StartsWith("Encompass", StringComparison.OrdinalIgnoreCase))
                    {
                        AutomationElement cmbBoxUserid = Utils2.FindElement(frm, "cmbBoxUserid");
                        if (cmbBoxUserid != null)
                        {
                            Utils2.SetText(cmbBoxUserid, sServer);
                            AutomationElement btnLogin = Utils2.FindElement(frm, "btnLogin");
                            Utils2.InvokeClick(btnLogin);
                            ClientIDDialog_Handled = true;
                            return true;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError(ex);
            }

            // fallthrough - not handled
            return false;
        }

        // return TRUE if we login
        private bool CheckForLoginDialog(AutomationElement frm)
        {
            // don't throw errors
            try
            {
                if (!LoginDialog_Handled)
                {
                    if (frm.Current.AutomationId.Equals("ThinThickForm", StringComparison.OrdinalIgnoreCase) &&
                        frm.Current.Name.StartsWith("Encompass", StringComparison.OrdinalIgnoreCase))
                    {
                        // find "webBrowser1"
                        AutomationElement webBrowser1 = Utils2.FindElement(frm, "webBrowser1");

                        // find "instanceId", "userId", "password", "Log In"
                        AutomationElement instanceId = Utils2.FindElement(webBrowser1, "Instance ID");
                        AutomationElement userId = Utils2.FindElement(webBrowser1, "User ID");
                        AutomationElement password = Utils2.FindElement(webBrowser1, "password");

                        // set values
                        Utils2.SetText(instanceId, sServer);
                        Utils2.SetText(userId, sUID);

                        if (!String.IsNullOrEmpty(sPWD))
                        {
                            Utils2.SetText(password, sPWD);
                            AutomationElement btn = Utils2.FindElement(webBrowser1, "Log In");
                            Utils2.InvokeClick(btn);
                        }

                        LoginDialog_Handled = true;
                        return true;
                    }
                }
            }
            catch (Exception ex)
            {
                HandleError(ex);
            }

            // fallthrough - not logged in
            return false;
        }

        private bool CheckForMainWindow(AutomationElement frm)
        {
            // don't throw errors
            try
            {
                if (frm.Current.AutomationId.Equals("MainForm", StringComparison.OrdinalIgnoreCase))
                {
                    return true;
                }
            }
            catch (Exception ex)
            {
                HandleError(ex);
            }

            // fallthrough - not logged in
            return false;
        }

        private void HandleError(Exception ex)
        {
            if (ex is ElementNotAvailableException)
            {
                // ignore - form disappeared
            }
            else
            {
                lblError.Text = ex.Message;
            }
        }

    }
}
